/*
 * @Description: 工单相关
 * @Author: Ben Lin
 * @version:
 * @Date: 2024-06-19 20:34:27
 * @LastEditors: Ben Lin
 * @LastEditTime: 2024-07-22 16:38:10
 */

import { Ref, h, ref, render, unref } from 'vue';
import { GetEnum, SaveEntity, convertToTree, getEntity } from '/@/api/tigerapi/system';
import { useLocale } from '/@/locales/useLocale';
import { isNullOrEmpty, isNullOrUnDef } from '/@/utils/is';
import { useI18n } from '/@/hooks/web/useI18n';
import { NavItem } from '/@/api/tigerapi/model/basModel';
import { useMessage } from '/@/hooks/web/useMessage';
import { useModal } from '/@/components/Modal';
import { ActionItem, BasicColumn, FormSchema } from '/@/components/Table';
import { useUserStore } from '/@/store/modules/user';
import { buildUUID } from '/@/utils/uuid';
import { SaveWoBatchInput } from '/@/api/tigerapi/model/mesModel';
import { CustModalParams, FunctionType } from '/@/api/tigerapi/model/systemModel';
import { SaveMesBatchWo } from '/@/api/tigerapi/mes/wo';
import { useForm } from '/@/components/Form/index';
import { uploadApi } from '/@/api/sys/upload';
import { initRoute } from '../data';
import { SP_MES_PROD2WO } from '/@/api/tigerapi/mes/router';
import { cloneDeep } from 'lodash-es';

const { t } = useI18n();
const { createErrorModal } = useMessage();
const { getLocale } = useLocale();
function _default() {
  /* 定义变量 */
  const isNormal = (type: number) => type === 0;
  const isScan = (type: number) => type === 1;
  const isAssy = (type: number) => type === 2;
  const isTest = (type: number) => type === 3;
  const isAudit = (type: number) => type === 4;
  const isPrint = (type: number) => type === 5;
  const isPackage = (type: number) => type === 6;
  const ActionColumn: BasicColumn = {
    width: 180,
    title: '操作',
    dataIndex: 'action',
    slots: { customRender: 'action' },
    fixed: 'right',
  };
  /**
   * @description: 工单管理的一些自定义方法
   * @return {*}
   */
  const methods = {
    /**
     * @description: 获取新增按钮的行为
     * @return {*}
     */
    CreateAction: (fnName: string) => {
      return {
        action: 'drawer', //drawer(打开左侧抽屉框) | go(跳转到新的页面)
      };
    },
    /**
     * @description: 操作字段自定义按钮
     * @return {*}
     */
    ActionItem: (params: Recordable<any>, data, ...args): ActionItem[] => {
      data.map((x) => {
        if (x.name == 'goDetail') {
          x.onClick = goDetail.bind(null, args[5], params);
          x.tooltip = '工单批次';
        }
        if (x.name == 'handleConfig') {
          x.onClick = handleConfig.bind(null, args, params);
          x.tooltip = '配置工艺';
        }
        if (x.name == 'handleRelease') {
          x.onClick = handleRelease.bind(null, args, params);
          x.tooltip = '下发';
        }
      });
      return data;
    },
    GetUseForm: () => {
      return {
        forminfo: useForm({
          labelWidth: 120,
          schemas: woformSchema,
          actionColOptions: {
            span: 24,
          },
          showActionButtonGroup: false,
        }),
        woinfo: useForm({
          labelWidth: 120,
          schemas: woCfgformSchema,
          actionColOptions: {
            span: 24,
          },
          showActionButtonGroup: false,
        }),
        prodinfo: useForm({
          labelWidth: 120,
          schemas: prodCfgformSchema,
          actionColOptions: {
            span: 24,
          },
          showActionButtonGroup: false,
        }),
      };
    },
    /**
     * @description: 自定义方法
     * @param {string} type
     * @param {array} args
     * @return {*}
     */
    CustFunc: (param: CustModalParams) => {
      switch (param.cType) {
        case 'BIZ_MES_WO':
        case 'BIZ_MES_WO_Config':
          return getWoFns[param.FnName](param) as Promise<any>;
        // break;
        default:
          return new Promise((resolve, reject) => {
            try {
              (e) => {};
              resolve(true);
            } catch {
              reject(false);
            } finally {
              // args[0][0]({ confirmLoading: false });
            }
          });
        // break;
      }
    },
    /**
     * @description: 获取多表单插槽列表
     * @return {*}
     */
    GetDtlSlots: (type: string) => {
      let result = [] as any[];
      switch (type) {
        case 'BIZ_MES_WO_Config':
          result = [
            { name: 'woinfo', slots: [], preIcons: {}, title: '工单信息' },
            {
              name: 'prodinfo',
              slots: ['addRot', 'setRot'],
              preIcons: { addRot: 'search|svg', setRot: 'config|svg' },
              title: '产品信息',
            },
            { name: 'rotinfo', slots: ['add'], preIcons: { add: 'search|svg' }, title: '工艺信息' },
          ];
          break;
        case 'BIZ_MES_WO':
          result = [
            {
              name: 'forminfo',
              slots: ['add', 'set'],
              preIcons: { add: 'search|svg', set: 'config|svg' }, //如果是多个表单，增加多个插槽。slots是弹出框按钮的插槽，preIcons是插槽对应的按钮图标
              title: '下发配置',
            },
          ];
          break;
      }
      return result;
    },
    /**
     * @description: 获取模态框应用列表
     * @return {*}
     */
    GetUseModals: () => {
      return {
        useModalData: {
          add: useModal(),
          set: useModal(),
          addRot: useModal(),
          setRot: useModal(),
        },
      };
    },
    /**
     * @description: 获取自定义数据
     * @return {*}
     */
    GetCustData: () => {
      return {
        isCustEl: {
          forminfo: false,
          woinfo: false,
          prodinfo: false,
          rotinfo: true,
        },
      };
    },
    /**
     * @description: 弹出选择框选择成功返回方法
     * @param {*} d
     * @param {*} u
     * @return {*}
     */
    GetSelectSuccess: (d, u) => {
      return {
        ROUTE_CODE: d.values['val'],
        // ID: d.values['id'],
        ROT_ID: d.values['id'],
      };
    },
    /**
     * @description: 打开弹出选择框
     * @param {Fn} openItemModal
     * @param {array} args
     * @return {*}
     */
    OpenSelectItem: (openItemModal: Fn, ...args) => {
      const slotName = args[0];
      const openRvModal = args[1];
      const selectVals = args[2];
      switch (slotName) {
        case 'addRot':
        case 'add':
          openItemModal(true, {
            title: '工艺路线列表',
            schemas: [
              {
                field: 'ROT_CODE',
                component: 'Input',
                label: '工艺路线编码',
                colProps: {
                  span: 12,
                },
              },
            ],
            ItemColumns: [
              {
                title: t('工艺路线编码'),
                dataIndex: 'ROT_CODE',
                resizable: true,
                sorter: true,
                width: 200,
              },
              {
                title: t('工艺路线名称'),
                dataIndex: 'ROT_NAME',
                resizable: true,
                sorter: true,
                width: 180,
              },
            ],
            tableName: 'MES_ROUTE',
            rowKey: 'ROT_CODE',
            returnFieldName: ['ROUTE_CODE', 'ROT_ID'], //返回值要赋值的字段名称
            searchInfo: { TABLE_NAME: 'MES_ROUTE' },
          });
          break;
        case 'set':
          openRvModal(true, { rotId: selectVals.value['ROUTE_CODE'], slotName: slotName });
          break;
        case 'setRot':
          break;
      }
    },
  };

  /* 以下是内部方法，不export，供上面的方法调用 */

  /**
   * @description: 自定义方法
   * @return {*}
   */
  const getWoFns: Record<string, FunctionType> = {
    /**
     * @description: 工单配置保存方法
     * @param {*} e
     * @param {array} args
     * @return {*}
     */
    SaveCofig: (param: CustModalParams) => {
      return new Promise((resolve, reject) => {
        try {
          const form = param.values['prodinfo'];
          const wo = param.values['mValues'];
          const _wo = cloneDeep(wo);
          _wo.ROUTE_STATUS = 1;
          _wo.ROUTE_CODE = form.ROUTE_CODE;
          SaveEntity(_wo, true, 'BIZ_MES_WO').then((action) => {
            if (action.IsSuccessed) {
              SP_MES_PROD2WO({ rotId: form.ROT_ID, wo: wo.ORDER_NO }).then((res) => {
                if (!res.IsSuccessed) {
                  SaveEntity(wo, true, 'BIZ_MES_WO');
                }
                resolve(res);
              });
            } else {
              reject(action);
            }
          });
        } catch {
          reject(false);
        }
      });
    },
    /**
     * @description: 工单下发保存方法
     * @param {CustModalParams} param
     * @return {*}
     */
    SaveWoBatch: (param: CustModalParams) => {
      return new Promise((resolve, reject) => {
        try {
          const form = param.values['forminfo'];
          const wo = param.values['mValues'];
          wo.ROUTE_CODE = form.ROUTE_CODE;
          wo.STATUS = 2;
          let input: SaveWoBatchInput = {
            Wo: wo,
            WoBatch: {
              ID: buildUUID(),
              CREATE_TIME: new Date(),
              CREATE_USER: useUserStore().getUserInfo.userId as string,
              UPDATE_TIME: new Date(),
              UPDATE_USER: useUserStore().getUserInfo.userId as string,
              GHOST_ROW: false,
              AUTH_ORG: useUserStore().getUserInfo.orgCode,
              AUTH_PROD: useUserStore().getUserInfo.prodCode as string,
              AUTH_WH: '',
              ORDER_NO: wo.ORDER_NO,
              STATUS: wo.STATUS,
              ITEM_CODE: wo.ITEM_CODE,
              CUST_CODE: wo.CUST_CODE,
              FACTORY: wo.FACTORY,
              WS_CODE: wo.WS_CODE,
              ACT_LINE: form.PLAN_LINE,
              STD_WORKER_QTY: wo.STD_WORKER_QTY,
              ACT_WORKER_QTY: wo.ACT_WORKER_QTY,
              RELEASE_TIME: new Date(),
              RELEASE_USER: useUserStore().getUserInfo.userId as string,
              PLAN_QTY: form.RELEASE_QTY,
              INPUT_QTY: wo.INPUT_QTY,
              OUTPUT_QTY: wo.OUTPUT_QTY,
              SCRAP_QTY: wo.SCRAP_QTY,
              STOCK_IN_QTY: wo.STOCK_IN_QTY,
              UPH: wo.UPH,
              UPPH: wo.UPPH,
              REMARK: wo.REMARK,
              ACT_START_TIME: wo.ACT_START_TIME,
              ACT_END_TIME: wo.ACT_END_TIME,
              BATCH_NO: '',
            },
          };
          SaveMesBatchWo(input).then((action) => {
            resolve(action);
          });
        } catch {
          reject(false);
        }
      });
    },
    initRoute: () => {},
  };

  /**
   * @description: 跳转到详情页面方法
   * @param {Fn} go
   * @return {*}
   */
  function goDetail(go: Fn, params: Recordable) {
    const id = {
      ID: params['record'].ID,
      CODE: params['record']['ORDER_NO'],
      Name: 'BIZ_MES_WO_BATCH',
      firstTabName: '工单批次',
      secondTabName: '', //'标签过程变量',
      firstTitle: '批次',
      secondTitle: '', //'过程变量',
      pageTitle: '工单详情', //详情页面标题
      contentStr: '这里是工单批次管理页面，可以显示工单下发的所有批次详情',
      detailName: `工单[${params['record'].ORDER_NO}]批次详情`,
      keyFieldValues: { ORDER_NO: params['record']['ORDER_NO'] },
      colSlots: params['colSlots'].value,
      SessionName: 'BIZ_MES_WO_BATCH_update',
    };
    // 将对象转换为JSON字符串并保存到sessionStorage
    sessionStorage.removeItem(`${id.SessionName}_params`);
    sessionStorage.setItem(`${id.SessionName}_params`, encodeURI(JSON.stringify(id)));
    go(`/BIZ_MES_WO_BATCH/${encodeURI(JSON.stringify({ sName: id.SessionName, Name: id.Name }))}`);
  }

  /**
   * @description: 配置工艺
   * @param {Recordable} record
   * @param {array} args
   * @return {*}
   */
  function handleConfig(args, params: Recordable) {
    // const openCustModal = args[7];
    // // rotSchema.value = getFormSchema('rotinfo');
    // params['routeData'].value = {
    //   nodes: [],
    //   edges: [],
    // };
    // initRoute(params['currlf'], params['record'].ROUTE_CODE, params['routeData'], params['currlf']);
    // getEntity({
    //   sqlcmd: `ROT_CODE = '${params['record'].ROUTE_CODE}'`,
    //   entityName: 'MES_ROUTE',
    //   order: '',
    // }).then((rot) => {
    //   params['selectVals'].value['ROUTE_CODE'] = params['record'].ROUTE_CODE;
    //   params['selectVals'].value['ROT_ID'] =
    //     !isNullOrEmpty(rot.Data) && !isNullOrEmpty(rot.Data.Items) ? rot.Data.Items[0].ID : '';
    //   openCustModal(true, {
    //     isUpdate: true,
    //     ctype: 'BIZ_MES_WO_Config',
    //     title: '工艺配置',
    //     width: '1200px',
    //     formEl: params['useFormData'].value, //如果是多个表单，增加多个插槽
    //     formElName: ['woinfo', 'prodinfo', 'rotinfo'], //表单插槽名称，支持多个表单
    //     RowKey: '',
    //     fnName: 'SaveCofig', //保存方法名
    //     others: params['routeData'].value,
    //     values: { ...params['record'], ...params['selectVals'].value }, //表单记录
    //   });
    // });
    const go = args[5];
    const id = {
      ID: params['record'].ID,
      Name: 'WoRouteBinding',
      CODE: params['record'].ORDER_NO,
      Title: `工单[${params['record'].ORDER_NO}]工艺绑定`,
      colSlots: params['colSlots'].value, /* 表格内的查询表单字段的插槽列表，一般用于弹出选择框按钮 */
      crudColSlots: [] /* 增删改表单字段的插槽列表，一般用于弹出选择框按钮 */,
      dense: true,
      pageTitle: `工单工艺绑定`,
      pageContent: `这里是管理工单的工艺绑定，一个工单可以绑定多个工艺路线`,
      SessionName: 'WoRouteBinding_update',
    };
    // 将对象转换为JSON字符串并保存到sessionStorage
    sessionStorage.removeItem(`${id.SessionName}_params`);
    sessionStorage.setItem(`${id.SessionName}_params`, encodeURI(JSON.stringify(id)));
    go(`/WoRouteBinding/CP/${encodeURI(JSON.stringify({ sName: id.SessionName, Name: id.Name }))}`);
  }

  /**
   * @description: 下发
   * @param {*} record
   * @return {*}
   */
  function handleRelease(args, params: Recordable) {
    const openCustModal = args[7];
    openCustModal(true, {
      isUpdate: true, //是否更新操作
      ctype: 'BIZ_MES_WO', //是哪个页面
      title: '工单下发', //标题
      width: '900px', //弹出框宽度
      formEl: params['useFormData'].value,
      formElName: ['forminfo'], //表单插槽名称
      RowKeys: { add: 'ROUTE_CODE', set: 'ROUTE_CODE' }, //插槽的弹出框选择的code
      fnName: 'SaveWoBatch', //保存方法名
      initFnName: {}, //初始化方法名，没有就留空{}
      values: params['record'], //表单记录
    });
  }

  const woformSchema: FormSchema[] = [
    {
      field: 'PLAN_QTY',
      label: '工单数量',
      component: 'Input',
      required: true,
      colProps: {
        span: 24,
      },
      dynamicDisabled: ({ values }) => {
        return true;
      },
    },
    {
      field: 'RELEASE_QTY',
      label: '工单下发数量',
      component: 'Input',
      required: true,
      colProps: {
        span: 24,
      },
    },
    {
      label: '计划产线',
      field: 'PLAN_LINE',
      component: 'ApiSelect',
      required: true,
      colProps: {
        span: 24,
      },
      componentProps: {
        api: getEntity,
        params: { entityName: 'MES_LINE', sqlcmd: ' 1=1 ', order: '' },
        resultField: 'Data.Items',
        labelField: 'LINE_NAME',
        valueField: 'LINE_CODE',
      },
    },
    {
      label: '工艺路线',
      field: 'ROUTE_CODE',
      component: 'Input',
      required: true,
      colProps: {
        span: 22,
      },
    },
    {
      field: 'add',
      component: 'Input',
      label: '1',
      colSlot: 'add',
      defaultValue: 'MES_ROUTE',
      colProps: {
        span: 1,
      },
    },
    {
      field: 'set',
      component: 'Input',
      label: '1',
      colSlot: 'set',
      defaultValue: 'MES_ROUTE',
      colProps: {
        span: 1,
      },
    },
  ];

  const woCfgformSchema: FormSchema[] = [
    {
      field: 'ORDER_NO',
      component: 'Input',
      label: '工单号',
      colProps: {
        span: 8,
      },
      componentProps: {
        placeholder: '请输入工单号',
        onChange: (e: any) => {
          console.log(e);
        },
      },
      dynamicDisabled: true,
    },
    {
      field: 'ORDER_TYPE',
      component: 'ApiSelect',
      label: '工单类型',
      colProps: {
        span: 8,
      },
      defaultValue: 0,
      componentProps: {
        api: GetEnum,
        params: { name: 'BIZ_MES_WO+ORDER_TYPEs' },
        resultField: 'Data',
        labelField: unref(getLocale) == 'zh_CN' ? 'Desc' : 'Name',
        valueField: 'Value',
        placeholder: '请选择工单类型',
      },
      dynamicDisabled: true,
    },
    {
      field: 'PLAN_QTY',
      component: 'Input',
      label: '计划数量',
      colProps: {
        span: 8,
      },
      dynamicDisabled: true,
    },
  ];

  const prodCfgformSchema: FormSchema[] = [
    {
      field: 'ITEM_CODE',
      component: 'Input',
      label: '物料编码',
      required: true,
      colProps: {
        span: 12,
      },
      componentProps: {
        placeholder: '请输入物料编码',
        onChange: (e: any) => {
          console.log(e);
        },
      },
      dynamicDisabled: true,
    },
    {
      field: 'ITEM_TYPE',
      component: 'Input',
      label: '物料类型',
      colProps: {
        span: 12,
      },
    },
    {
      label: '工艺路线',
      field: 'ROUTE_CODE',
      component: 'Input',
      required: true,
      colProps: {
        span: 12,
      },
    },
    {
      field: 'addRot',
      component: 'Input',
      label: '1',
      colSlot: 'addRot',
      defaultValue: 'MES_ROUTE',
      colProps: {
        span: 1,
      },
    },
    {
      field: 'setRot',
      component: 'Input',
      label: '1',
      colSlot: 'setRot',
      defaultValue: 'MES_ROUTE',
      colProps: {
        span: 1,
      },
    },
    {
      label: '工艺路线',
      field: 'ROT_ID',
      component: 'Input',
      show: false,
      colProps: {
        span: 12,
      },
    },
    // {
    //   field: 'fieldsc',
    //   component: 'Upload',
    //   label: '打印模板',
    //   colProps: {
    //     span: 10,
    //   },
    //   rules: [{ required: true, message: '请选择上传文件' }],
    //   componentProps: {
    //     api: uploadApi,
    //     multiple: false,
    //   },
    // },
  ];

  return [methods, ActionColumn];
}

export default _default;
